#include "StdAfx.h"
#include "SimpleEdit.h"

#include "ScriptletConstant.h"
#include "FullFillLayoutManager.h"

#define  _ATL_TMP_NO_CSTRING
#include <atlctrls.h>

class EditCtrl : public CNWindowAdapter
{
public:
	EditCtrl(void) {}
	~EditCtrl(void) {}

	//override
	HWND CreateExWindow( LPTSTR szClassName, DWORD dwStyle, HWND hWndParent, UINT uID, DWORD dwStyleEx )
	{
		if (m_Edit.IsWindow())
		{
			return m_Edit.m_hWnd;
		}
		HWND hWnd = m_Edit.Create(hWndParent, CRect(0, 0, 0, 0), _T(""), dwStyle, dwStyleEx, uID);
		if (IsWindow(hWnd))
		{
			this->AttachWindowAdaptee(hWnd);
		}
		return hWnd;
	}

	//operation
	void SetText(LPCTSTR text)
	{
		if (m_Edit.IsWindow())
		{
			CString oldText;
			m_Edit.GetWindowText(oldText);
			if (oldText.Compare(text) != 0)
			{
				m_Edit.SetWindowText(text);
			}
		}
	}
	BOOL GetText(CString& text)
	{
		if (m_Edit.IsWindow())
		{	
			m_Edit.GetWindowText(text);
		}
		return TRUE;
	}
	void SetFocus()
	{
		if (m_Edit.IsWindow())
		{
			m_Edit.SetFocus();
		}
	}
	void SetSel(int start, int end)
	{
		if (m_Edit.IsWindow())
		{
			m_Edit.SetSel(start, end);
		}
	}
	void SetLimitText(int maxText)
	{
		if (m_Edit.IsWindow())
		{
			m_Edit.SetLimitText(maxText);
		}
	}
	VBEGIN_MSG_MAP(SetPathEditPanel)
		COMMAND_CODE_HANDLER(EN_CHANGE, OnEnChange)
		CHAIN_MSG_MAP(CNWindowAdapter)
	END_MSG_MAP()

	DECLARE_COMMAND_HANDLER(OnEnChange)
	{
		SimpleEdit* edit = dynamic_cast<SimpleEdit*>(GetParent());
		if (edit)
		{			
			if (edit->IsChanged())
			{
				edit->OnEnChange();
			}
		}
		return 0;
	}
private:
	CEdit m_Edit;
};

REGISTER_COMPONENT_CREATOR(SCRIPTLET_SIMPLEEDIT_NODE, SimpleEdit);
SimpleEdit::SimpleEdit(void)
{
	m_Edit = NULL;
	m_bMultiline = FALSE;
	m_bReadOnly = FALSE;
	// [:2008-12-18] Memory Leak  : SetLayoutManager AutoReleaseϱ ؼ Ʒ  
	//this->SetLayoutManager(new CFullFillLayoutManager());
	CFullFillLayoutManager* layoutManager = new CFullFillLayoutManager();
	this->SetLayoutManager(layoutManager);
	layoutManager->AutoRelease();
}

SimpleEdit::~SimpleEdit(void)
{
}
HFONT SimpleEdit::CreateTextFont(HDC hDC)
{
	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));
	lf.lfWidth = 0;
	lf.lfOrientation = 0;
	lf.lfCharSet = m_cf.bCharSet;
	lf.lfOutPrecision = OUT_CHARACTER_PRECIS;
	lf.lfHeight = -MulDiv(m_cf.yHeight/20, GetDeviceCaps(hDC, LOGPIXELSY), 72);
	lf.lfWeight = m_cf.dwEffects & CFE_BOLD ? FW_BOLD : FW_NORMAL;
	lf.lfItalic = m_cf.dwEffects & CFE_ITALIC ? TRUE : FALSE;
	_tcscpy(lf.lfFaceName, m_cf.szFaceName);

	HFONT hFont = ::CreateFontIndirect(&lf);
	return hFont;
}
void SimpleEdit::OnAddNotify()
{
	__super::OnAddNotify();

	if (m_Edit == NULL)
	{
		m_Edit = new EditCtrl;
		m_Edit->CreateExWindow(NULL, WS_CHILD | WS_VISIBLE | WS_TABSTOP | ES_AUTOHSCROLL | WS_CLIPCHILDREN | WS_CLIPSIBLINGS
			| (m_bMultiline==TRUE ? ES_MULTILINE | ES_AUTOVSCROLL | WS_VSCROLL: 0)
			| (m_bReadOnly ? ES_READONLY : 0), 
			this->GetOwnerWindow(), 0 , 0);
		HDC hDC = ::GetDC(NULL);
		if (hDC != NULL)
		{
			HFONT font = CreateTextFont(hDC);
			CWindow window = m_Edit->GetWindowAdaptee();
			window.SetFont(font);
			ReleaseDC(NULL, hDC);
		}
		Add(m_Edit);
	}	
}
void SimpleEdit::OnRemoveNotify()
{
	__super::OnRemoveNotify();
	if (m_Edit)
	{
		Remove(m_Edit);
	}
}
void SimpleEdit::SetFocus()
{
	if (m_Edit)
	{
		m_Edit->SetFocus();
	}
}
LPCTSTR SimpleEdit::GetText()
{
	if (m_Edit)
	{
		m_Edit->GetText(m_strText);
		return m_strText;
	}
	return __super::GetText();
}
void SimpleEdit::SetText(LPCTSTR text)
{
	if (m_Edit)
	{
		m_Edit->SetText(text);	
		return;
	}
	__super::SetText(text);
}
BOOL SimpleEdit::IsChanged()
{
	if (m_Edit)
	{
		CString text;
		m_Edit->GetText(text);

		return (_tcscmp(text, __super::GetText()));
	}
	return FALSE;
}
void SimpleEdit::SetSel(int start, int end)
{
	if (m_Edit)
	{
		m_Edit->SetSel(start, end);	
	}
}
void SimpleEdit::SetLimitText(int maxText)
{
	if (m_Edit)
	{
		m_Edit->SetLimitText(maxText);
	}
}
///////////////////////////////////////////////////////////////////////////
//SimpleEditScriptPlugin
//////////////////////////////////////////////////////////////////////////
REGISTER_SCRIPTLETCOMPONENTPLUGIN_CREATOR(SCRIPTLET_SIMPLEEDIT_NODE, SimpleEditScriptPlugin);
SimpleEditScriptPlugin::SimpleEditScriptPlugin(void)
{
}

SimpleEditScriptPlugin::~SimpleEditScriptPlugin(void)
{
}

BOOL SimpleEditScriptPlugin::ModifyComponent(CContainer* pParent, CComponent* pTarget, TiXmlElement* pNode)
{	
	BOOL bSuccess = __super::ModifyComponent(pParent, pTarget, pNode);
	if (bSuccess)
	{
		SimpleEdit* simpleedit = reinterpret_cast<SimpleEdit*>(pTarget);
		if (simpleedit == NULL)
		{
			return FALSE;
		}
		TCHAR szValue[SCRIPTLET_MAX_VALUE] = {0};

		if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_TEXT_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_TEXT_NODE), szValue, SCRIPTLET_MAX_VALUE))
		{
			simpleedit->SetText(szValue);
		}
		if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_MULTILINE_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_MULTILINE_NODE), szValue, SCRIPTLET_MAX_VALUE))
		{
			BOOL bMultiline = _tcsicmp(szValue, SCRIPTLET_TRUE_VALUE) == 0;
			simpleedit->SetMultiline(bMultiline);
		}
		if (XMLUtil::GetAttrValue(pNode, CT2CA(L"readonly"), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(L"readonly"), szValue, SCRIPTLET_MAX_VALUE))
		{
			BOOL bReadonly = _tcsicmp(szValue, SCRIPTLET_TRUE_VALUE) == 0;
			simpleedit->SetReadOnly(bReadonly);
		}
	}
	return bSuccess;
}
CComponent* SimpleEditScriptPlugin::_CreateComponent(TiXmlElement* pNode)
{
	CComponent* pComponent = new SimpleEdit;
	return pComponent;
}
//////////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////////
REGISTER_PROPERTY_ACCESSOR(SCRIPTLET_SIMPLEEDIT_NODE, SimpleEditPropertyAccessor);

BOOL SimpleEditPropertyAccessor::SetProperty(CComponent* pTarget, LPCTSTR property, CComVariant value)
{
	BOOL bSuccess = __super::SetProperty(pTarget, property, value);
	if (bSuccess == FALSE)
	{
		SimpleEdit* simpleedit = reinterpret_cast<SimpleEdit*>(pTarget);
		if (simpleedit == NULL)
		{
			return bSuccess;
		}
		if (_tcsicmp(SCRIPTLET_TEXT_NODE, property) == 0)
		{
			simpleedit->SetText(CString(value.bstrVal));		
		}
		else if (_tcsicmp(SCRIPTLET_MULTILINE_NODE, property) == 0)
		{
			simpleedit->SetMultiline(value.boolVal);
		}
		else if (_tcsicmp(L"readonly", property) == 0)
		{
			simpleedit->SetReadOnly(value.boolVal);
		}
	}
	return bSuccess;
}
BOOL SimpleEditPropertyAccessor::GetProperty(CComponent* pTarget, LPCTSTR property, CComVariant& value)
{
	BOOL bSuccess = __super::GetProperty(pTarget, property, value);
	if (bSuccess == FALSE)
	{

	}
	return bSuccess;
}